<?php

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Permet d'obtenir le prix HT d'un objet SPIP avec une option
 * C'est le résultat de cette fonction qui est utilisée pour calculer le prix TTC.
 *
 * @param $objet
 * @param $id_objet
 * @param $id_option
 * @param $options
 * @param $serveur
 * @return array|float|int|mixed|string|null
 */
function inc_prix_option_ht_dist($objet, $id_objet, $id_option, $options = [], $serveur = '') {
	static $cache;
	$hash = md5(json_encode([$objet, $id_objet, $id_option, $options, $serveur]));
	if (isset($cache[$hash])) {
		return $cache[$hash];
	}
	include_spip('base/objets');
	include_spip('prix_fonctions'); // pas chargé automatiquement dans certains cas
	include_spip('intl_fonctions'); // pas chargé automatiquement dans certains cas

	$prix_ht = 0;

	// Arrondi de la devise
	$devise = intl_devise_defaut();
	$arrondi_devise = intval(intl_devise_info($devise, 'fraction'));

	// Compatibilité avec l'ancienne signature
	if (is_int($options)) {
		$options = [
			'arrondi' => $options,
			'serveur' => $serveur,
		];
	}
	// Options par défaut
	$options_defaut = [
		'arrondi' => $arrondi_devise,
		'serveur' => '',
	];
	// On fusionne avec les défauts
	$options = array_merge($options_defaut, $options);

	// Cherchons d'abord si l'objet existe bien
	if ($objet
		and $id_objet = intval($id_objet)
		and include_spip('base/connect_sql')
		and $objet = objet_type($objet)
		and $table_sql = table_objet_sql($objet, $serveur)
		and $cle_objet = id_table_objet($objet, $serveur)
		and $ligne = sql_fetsel('*', $table_sql, "$cle_objet = $id_objet", '', '', '', '', $serveur)
	) {

		// Existe-t-il une fonction précise pour le prix HT de ce type d'objet : prix_option_<objet>_ht() dans prix_option/<objet>.php
		if ($fonction_ht = charger_fonction('ht', "prix_option/$objet", true)) {
			// On passe la ligne SQL en paramètre pour ne pas refaire la requête
			$prix_ht = $fonction_ht($objet, $id_objet, $id_option, $ligne);
		} else {
			// S'il n'y a pas de fonction, regardons s'il existe des champs normalisés, ce qui évite d'écrire une fonction pour rien
			if ($ligne['prix_ht']) {
				$prix_ht = $ligne['prix_ht'];
			} else if ($ligne['prix']) {
				$prix_ht = $ligne['prix'];
			}
			if ($id_option) {
				$prix_option_ht = sql_getfetsel(
					'prix_option_objet',
					'spip_options_liens',
					[
						'id_option = ' . $id_option,
						'objet = ' . sql_quote($objet),
						'id_objet = ' . $id_objet,
					]
				);
				$prix_ht += $prix_option_ht;
			}
		}

		// On passe dans un pipeline pour modifier le prix HT
		$prix_ht = pipeline(
			'prix_option_ht',
			array(
				'args' => array(
					'objet' => $objet,
					'id_objet' => $id_objet,
					'id_option' => $id_option,
					'type_objet' => $objet, // déprécié, utiliser plutôt "objet"
					'prix_ht' => $prix_ht,
					'options' => $options,
					'ligne' => $ligne,
				),
				'data' => $prix_ht
			)
		);
	}

	// Si on demande un arrondi, on le fait
	// Enfin si nécessaire, on fait un arrondi
	if (
		is_int($options['arrondi'])
		and $options['arrondi'] >= 0
	) {
		$prix_ht = round($prix_ht, $options['arrondi']);
	}

	$cache[$hash] = $prix_ht;

	return $prix_ht;
}


 /**
  * Permet d'obtenir le prix final TTC d'un objet SPIP avec une option
  *
 * @param $objet
 * @param $id_objet
 * @param $id_option
 * @param $options
 * @param $serveur
 * @return array|float|mixed|null
 */
 function inc_prix_option_dist($objet, $id_objet, $id_option, $options = [], $serveur = '') {
	 include_spip('base/objets');
	 include_spip('prix_fonctions'); // pas chargé automatiquement dans certains cas
	 include_spip('intl_fonctions'); // pas chargé automatiquement dans certains cas

	// Arrondi de la devise
	$devise = intl_devise_defaut();
	$arrondi_devise = intval(intl_devise_info($devise, 'fraction'));

	// Compatibilité avec l'ancienne signature
	if (is_int($options)) {
		$options = [
			'arrondi' => $options,
			'serveur' => $serveur,
		];
	}
	// Options par défaut
	$options_defaut = [
		'arrondi' => $arrondi_devise,
		'serveur' => '',
	];
	// On fusionne avec les défauts
	$options = array_merge($options_defaut, $options);

	// On va d'abord chercher le prix HT. On délègue le test de présence de l'objet dans cette fonction.
	$fonction_prix_ht = charger_fonction('ht', 'inc/prix_option');
	$objet = objet_type($objet);
	$options_ht = array_merge($options, ['arrondi' => false]);
	$prix = $prix_ht = $fonction_prix_ht($objet, $id_objet, $id_option, $options_ht);
	$taxes = [];

	// On cherche maintenant s'il existe une personnalisation pour le prix total TTC : prix_<objet>() dans prix/<objet>.php
	if ($fonction_prix_objet = charger_fonction($objet, 'prix_option/', true)) {
		$prix = $fonction_prix_objet($objet, $id_objet, $options, $prix_ht);
	}

	// Sinon on appelle une fonction générique pour trouver les taxes d'un objet, et on ajoute au HT
	else if ($fonction_taxes = charger_fonction('taxes', 'inc/', true)) {
		$taxes = $fonction_taxes($objet, $id_objet, $options);
		$total_taux = array_sum(array_column($taxes, 'taux'));
		$prix = $prix_ht * (1 + $total_taux);
	}

	 // On passe dans un pipeline pour pouvoir ajouter taxes, ristournes ou autres modifications
	 $prix = pipeline(
		 'prix_option',
		 array(
			 'args' => array(
				 'objet' => $objet,
				 'id_objet' => $id_objet,
				 'id_option' => $id_option,
				 'type_objet' => $objet, // déprécié, utiliser plutôt "objet"
				 'prix_ht' => $prix_ht,
				 'options' => $options,
				 'taxes' => $taxes,
			 ),
			 'data' => $prix
		 )
	 );

	// Enfin si nécessaire, on fait un arrondi
	if (
		is_int($options['arrondi'])
		and $options['arrondi'] >= 0
	) {
		$prix = round($prix, $options['arrondi']);
	}

	// Et c'est fini
	return $prix;
}

